@@ -39,7 +39,8 @@ StringArray::~StringArray()
 const apr_array_header_t *StringArray::array(const SVN::Pool &pool)
 {
   apr_array_header_t *strings
-    = apr_array_make(pool.getPool(), m_strings.size(), sizeof(char *));
+    = apr_array_make(pool.getPool(), static_cast<int>(m_strings.size()),
+                     sizeof(char *));
 
   std::vector<std::string>::const_iterator it;
   for (it = m_strings.begin(); it < m_strings.end(); ++it)
@@ -69,7 +70,7 @@ StringArray::init(void)
       if (JNIUtil::isExceptionThrown())
         return;
 
-      m_strings.push_back(std::string((const char *)str));
+      m_strings.push_back(std::string(static_cast<const char *>(str)));
     }
 }
 
